from qrunner.running.runner import main
from qrunner.case import TestCase, Page, H5Page
from qrunner.core.h5.driver import H5Driver
from qrunner.decorate import *
from qrunner.utils.log import logger


__version__ = "0.7.5"
__description__ = "全栈自动化测试框架"

